import 'dart:async';
import 'package:flutter/services.dart';
import 'package:intl/intl.dart';
import 'package:collection/collection.dart';
import 'dart:convert';

class Dapi {
  static const MethodChannel _channel = const MethodChannel('dapi');
  static const EventChannel _successfulConnectionChannel =
      const EventChannel('co.dapi.connect/connect_success_channel');
  static const EventChannel _failureConnectionChannel =
      const EventChannel('co.dapi.connect/connect_failure_channel');
  static const EventChannel _bankRequestChannel =
      const EventChannel('co.dapi.connect/bank_request_channel');
  static const EventChannel _connectDismissedChannel =
      const EventChannel('co.dapi.connect/connect_dismissed_channel');

  static const EventChannel _transferUiDismissedChannel =
      const EventChannel('co.dapi.connect/transfer_ui_dismissed_channel');

  static const EventChannel _transferUiWillSendChannel =
      const EventChannel('co.dapi.connect/transfer_ui_will_send_channel');

  static Stream<ConnectSuccessEvent> get onConnectionSuccessful =>
      _successfulConnectionChannel
          .receiveBroadcastStream()
          .map((event) => ConnectSuccessEvent._fromMap(event));

  static Stream<ConnectFailureEvent> get onConnectionFailure =>
      _failureConnectionChannel
          .receiveBroadcastStream()
          .map((event) => ConnectFailureEvent._fromMap(event));

  static Stream<BankRequestEvent> get onBankRequest => _bankRequestChannel
      .receiveBroadcastStream()
      .map((event) => BankRequestEvent._fromMap(event));

  static Stream<Null> get onConnectDismissed =>
      _connectDismissedChannel.receiveBroadcastStream().map((event) => event);

  static Stream<Null> get onTransferUiDismissed => _transferUiDismissedChannel
      .receiveBroadcastStream()
      .map((event) => event);

  static Stream<TransferUiWillSendEvent> get onTransferUiWillSend =>
      _transferUiWillSendChannel
          .receiveBroadcastStream()
          .map((event) => TransferUiWillSendEvent._fromMap(event));

  static Future start(String appKey, String clientUserID,
      {DapiConfigurations? configurations}) async {
    try {
      List<dynamic> params = <dynamic>[
        appKey,
        clientUserID,
        configurations?._toMap()
      ];
      return await _channel.invokeMethod(
          'startWithAppKey:clientUserID:configurations:', params);
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static void setConfigurations(DapiConfigurations configurations) async {
    try {
      List<dynamic> params = <dynamic>[configurations._toMap()];
      _channel.invokeMethod('setConfigurations:', params);
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<DapiConfigurations> configurations() async {
    try {
      final Map? mapResult = await _channel.invokeMethod('configurations');
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Null configurations"));
      } else {
        final DapiConfigurations configurations =
            DapiConfigurations._fromMap(mapResult);
        return configurations;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static void presentConnect({String? bankID}) {
    try {
      List<dynamic> params = <dynamic>[bankID];
      _channel.invokeMethod('presentConnect:', params);
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static void dismissConnect() {
    try {
      _channel.invokeMethod('dismissConnect');
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static void dismissTransfer() {
    try {
      _channel.invokeMethod('dismissTransfer');
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static void dismissMFA() {
    try {
      _channel.invokeMethod('dismissMFA');
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<bool> isStarted() async {
    try {
      final bool? isStarted = await _channel.invokeMethod('isStarted');
      return isStarted ?? false;
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static void setClientUserID(String clientUserID) async {
    try {
      List<dynamic> params = <dynamic>[clientUserID];
      _channel.invokeMethod('setClientUserID:', params);
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<String?> clientUserID() async {
    try {
      final String? clientUserID = await _channel.invokeMethod('clientUserID');
      return clientUserID;
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<List<DapiConnection>> getConnections() async {
    try {
      final List<dynamic>? jsonInstances =
          await _channel.invokeMethod('getConnections');

      if (jsonInstances == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Null connections"));
      } else {
        List<Map<String, dynamic>> mappedJsonInstances =
            jsonInstances.map((e) => Map<String, dynamic>.from(e)).toList();
        List<DapiConnection> connections =
            mappedJsonInstances.map((e) => DapiConnection._fromMap(e)).toList();
        return connections;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<DapiConnection> _createConnection(
      String jsonConnectionParams) async {
    try {
      List<dynamic> params = <dynamic>[jsonConnectionParams];
      final dynamic jsonInstance =
          await _channel.invokeMethod('createConnection:', params);
      List<DapiConnection?> connections = await getConnections();
      DapiConnection? connection = connections.firstWhereOrNull(
          (element) => element?.userID == jsonInstance['userID']);
      if (connection == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid bank connection"));
      } else {
        return connection;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<String> _getParameters(String userID) async {
    try {
      List<dynamic> params = <dynamic>[userID];
      final dynamic jsonInstance =
          await _channel.invokeMethod('getParameters:', params);
      String connectionParams = jsonInstance.toString();
      return connectionParams;
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<String> _presentAccountSelection(String userID) async {
    try {
      List<dynamic> params = <dynamic>[userID];
      final dynamic accountID =
          await _channel.invokeMethod('presentAccountSelection:', params);
      return accountID.toString();
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<DapiIdentityResponse> _getIdentity(String userID) async {
    try {
      List<dynamic> params = <dynamic>[userID];
      final Map? mapResult =
          await _channel.invokeMethod('getIdentity:', params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final DapiIdentityResponse identity =
            DapiIdentityResponse._fromMap(mapResult);
        return identity;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<DapiAccountsResponse> _getAccounts(String userID) async {
    try {
      List<dynamic> params = <dynamic>[userID];
      final Map? mapResult =
          await _channel.invokeMethod('getAccounts:', params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final DapiAccountsResponse accounts =
            DapiAccountsResponse._fromMap(mapResult);
        return accounts;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<DapiCardsResponse> _getCards(String userID) async {
    try {
      List<dynamic> params = <dynamic>[userID];
      final Map? mapResult = await _channel.invokeMethod('getCards:', params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final DapiCardsResponse cards = DapiCardsResponse._fromMap(mapResult);
        return cards;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<DapiTransactionsResponse> _getTransactionsForAccount(
      String userID,
      String accountID,
      DateTime fromDate,
      DateTime toDate,
      DapiTransactionsType type) async {
    String stringType;

    if (type == DapiTransactionsType.ENRICHED) {
      stringType = "enriched";
    } else if (type == DapiTransactionsType.CATEGORIZED) {
      stringType = "categorized";
    } else {
      stringType = "default";
    }

    final List<dynamic> params = [
      userID,
      accountID,
      fromDate.millisecondsSinceEpoch,
      toDate.millisecondsSinceEpoch,
      stringType
    ];
    try {
      final Map? mapResult = await _channel.invokeMethod(
          'getTransactionsForAccount:accountID:startDate:endDate:type:',
          params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final DapiTransactionsResponse transactions =
            DapiTransactionsResponse._fromMap(mapResult);
        return transactions;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<DapiTransactionsResponse> _getTransactionsForCard(
      String userID,
      String cardID,
      DateTime fromDate,
      DateTime toDate,
      DapiTransactionsType type) async {
    String stringType;

    if (type == DapiTransactionsType.ENRICHED) {
      stringType = "enriched";
    } else if (type == DapiTransactionsType.CATEGORIZED) {
      stringType = "categorized";
    } else {
      stringType = "default";
    }

    final List<dynamic> params = [
      userID,
      cardID,
      fromDate.millisecondsSinceEpoch,
      toDate.millisecondsSinceEpoch,
      stringType
    ];
    try {
      final Map? mapResult = await _channel.invokeMethod(
          'getTransactionsForCard:cardID:startDate:endDate:type:', params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final DapiTransactionsResponse transactions =
            DapiTransactionsResponse._fromMap(mapResult);
        return transactions;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<DapiAccountsMetadataResponse> _getAccountsMetadata(
      String userID) async {
    try {
      List<dynamic> params = <dynamic>[userID];
      final Map? mapResult =
          await _channel.invokeMethod('getAccountsMetadata:', params);

      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final DapiAccountsMetadataResponse accountsMetaData =
            DapiAccountsMetadataResponse._fromMap(mapResult);
        return accountsMetaData;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<DapiResult> _delete(String userID) async {
    try {
      List<dynamic> params = <dynamic>[userID];
      final Map? mapResult = await _channel.invokeMethod('delete:', params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final DapiResult result = DapiResult._fromMap(mapResult);
        return result;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<DapiBeneficiariesResponse> _getBeneficiaries(
      String userID) async {
    try {
      List<dynamic> params = <dynamic>[userID];
      final Map? mapResult =
          await _channel.invokeMethod('getBeneficiaries:', params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final DapiBeneficiariesResponse beneficiaries =
            DapiBeneficiariesResponse._fromMap(mapResult);
        return beneficiaries;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<DapiResult> _createBeneficiary(
      String userID, DapiBeneficiary beneficiary) async {
    try {
      final List<dynamic> params = [
        userID,
        beneficiary._toMap(),
      ];
      final Map? mapResult =
          await _channel.invokeMethod('createBeneficiary:beneficiary:', params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final DapiResult result = DapiResult._fromMap(mapResult);
        return result;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<CreateTransferResponse> _createTransfer(
    String userID,
    String? accountID,
    DapiBeneficiary? beneficiary,
    double? amount,
    String? remark,
  ) async {
    final List<dynamic> params = [
      userID,
      accountID,
      beneficiary?._toMap(),
      amount,
      remark
    ];
    try {
      final Map? mapResult = await _channel.invokeMethod(
          'createTransfer:accountID:beneficiary:amount:remark:', params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final CreateTransferResponse result =
            CreateTransferResponse._fromMap(mapResult);
        return result;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<CreateTransferResponse> _createTransferToExistingBeneficiary(
    String userID,
    String accountID,
    String receiverID,
    double amount,
    String? remark,
  ) async {
    final List<dynamic> params = [
      userID,
      accountID,
      receiverID,
      amount,
      remark
    ];
    try {
      final Map? mapResult = await _channel.invokeMethod(
          'createTransferToExistingBeneficiary:accountID:receiverID:amount:remark:',
          params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final CreateTransferResponse result =
            CreateTransferResponse._fromMap(mapResult);
        return result;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<DapiWireBeneficiariesResponse> _getWireBeneficiaries(
      String userID) async {
    try {
      List<dynamic> params = <dynamic>[userID];
      final Map? mapResult =
          await _channel.invokeMethod('getWireBeneficiaries:', params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final DapiWireBeneficiariesResponse beneficiaries =
            DapiWireBeneficiariesResponse._fromMap(mapResult);
        return beneficiaries;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<DapiResult> _createWireBeneficiary(
      String userID, DapiWireBeneficiary beneficiary) async {
    try {
      final List<dynamic> params = [
        userID,
        beneficiary._toMap(),
      ];
      final Map? mapResult = await _channel.invokeMethod(
          'createWireBeneficiary:beneficiary:', params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final DapiResult result = DapiResult._fromMap(mapResult);
        return result;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<CreateTransferResponse> _createWireTransfer(
    String userID,
    String? accountID,
    DapiWireBeneficiary beneficiary,
    double? amount,
    String? remark,
  ) async {
    final List<dynamic> params = [
      userID,
      accountID,
      beneficiary._toMap(),
      amount,
      remark
    ];
    try {
      final Map? mapResult = await _channel.invokeMethod(
          'createWireTransfer:accountID:beneficiary:amount:remark:', params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final CreateTransferResponse result =
            CreateTransferResponse._fromMap(mapResult);
        return result;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<CreateTransferResponse>
      _createWireTransferToExistingBeneficiary(
    String userID,
    String accountID,
    String receiverID,
    double amount,
    String? remark,
  ) async {
    final List<dynamic> params = [
      userID,
      accountID,
      receiverID,
      amount,
      remark
    ];
    try {
      final Map? mapResult = await _channel.invokeMethod(
          'createWireTransferToExistingBeneficiary:accountID:receiverID:amount:remark:',
          params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final CreateTransferResponse result =
            CreateTransferResponse._fromMap(mapResult);
        return result;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }

  static Future<CreateTransferResponse> _createACHPullTransfer(
    String userID,
    String description,
    String? accountID,
    double? amount,
  ) async {
    final List<dynamic> params = [
      userID,
      description,
      accountID,
      amount,
    ];
    try {
      final Map? mapResult = await _channel.invokeMethod(
          'createACHPullTransfer:description:accountID:amount:', params);
      if (mapResult == null) {
        throw DapiSdkException._fromError(
            PlatformException(code: "0", message: "Invalid response"));
      } else {
        final CreateTransferResponse result =
            CreateTransferResponse._fromMap(mapResult);
        return result;
      }
    } on PlatformException catch (ex) {
      throw DapiSdkException._fromError(ex);
    }
  }
}

class DapiConfigurations {
  DapiEnvironment environment;
  List<String> countries;
  @Deprecated(
      "Not usable anymore because Dapi backend accepts the default endpoints paths only now.")
  Map<String, String>? endpoints;
  Map<String, dynamic>? extraBody;
  Map<String, dynamic>? extraQueryParameters;
  Map<String, dynamic>? extraHeaderFields;
  bool? showLogos = true;
  bool? showExperimentalBanks = false;
  bool? showCloseButton = true;
  bool? showAddButton = false;
  bool? showTransferSuccessfulResult = false;
  bool? showTransferErrorResult = false;
  String? postSuccessfulConnectionLoadingText = "Verifying your account..";
  DapiThemeConfigurations? theme;
  DapiLanguage? language = DapiLanguage.EN;

  DapiConfigurations(
      {required this.environment,
      required this.countries,
      this.endpoints,
      this.extraBody,
      this.extraQueryParameters,
      this.extraHeaderFields,
      this.showLogos,
      this.showExperimentalBanks,
      this.showCloseButton,
      this.showAddButton,
      this.showTransferSuccessfulResult,
      this.showTransferErrorResult,
      this.postSuccessfulConnectionLoadingText,
      this.theme,
      this.language});

  Map<String, dynamic> _toMap() {
    return {
      "extraBody": this.extraBody,
      "extraQueryParameters": this.extraQueryParameters,
      "extraHeaderFields": this.extraHeaderFields,
      "environment": this.environment.toString().split('.').last.toLowerCase(),
      "countries": this.countries,
      "showLogos": this.showLogos,
      "showExperimentalBanks": this.showExperimentalBanks,
      "showCloseButton": this.showCloseButton,
      "showAddButton": this.showAddButton,
      "showTransferSuccessfulResult": this.showTransferSuccessfulResult,
      "showTransferErrorResult": this.showTransferErrorResult,
      "postSuccessfulConnectionLoadingText":
          this.postSuccessfulConnectionLoadingText,
      "theme": {
        "enforceTheme":
            this.theme?.enforceTheme.toString().split('.').last.toLowerCase(),
        "primaryColor": {
          "lightMode": this.theme?.primaryColor?.lightMode,
          "darkMode": this.theme?.primaryColor?.darkMode
        }
      },
      "language": this.language.toString().split('.').last.toLowerCase()
    };
  }

  static DapiConfigurations _fromMap(Map map) {
    DapiEnvironment environment;
    DapiLanguage language;

    String environmentRaw = map['environment'].toString().toLowerCase();
    String languageRaw = map['language'].toString().toLowerCase();

    if (environmentRaw == "sandbox") {
      environment = DapiEnvironment.SANDBOX;
    } else {
      environment = DapiEnvironment.PRODUCTION;
    }

    if (languageRaw == "ar") {
      language = DapiLanguage.AR;
    } else {
      language = DapiLanguage.EN;
    }

    List<Object?> countriesRaw = map['countries'];

    List<String> countries = List.empty(growable: true);

    countriesRaw.forEach((element) {
      countries.add(element.toString());
    });

    Map<dynamic, dynamic>? extraBodyRaw = map['extraBody'];
    Map<String, dynamic>? extraBody = {};
    extraBodyRaw?.forEach((key, value) {
      extraBody[key.toString()] = value;
    });

    Map<dynamic, dynamic>? extraHeadersRaw = map['extraHeaderFields'];
    Map<String, dynamic>? extraHeaders = {};
    extraHeadersRaw?.forEach((key, value) {
      extraHeaders[key.toString()] = value;
    });

    Map<dynamic, dynamic>? extraQueryParametersRaw =
        map['extraQueryParameters'];
    Map<String, dynamic>? extraQueryParameters = {};
    extraQueryParametersRaw?.forEach((key, value) {
      extraQueryParameters[key.toString()] = value;
    });

    return DapiConfigurations(
        environment: environment,
        countries: countries,
        extraBody: extraBody,
        extraHeaderFields: extraHeaders,
        extraQueryParameters: extraQueryParameters,
        showLogos: map['showLogos'],
        showExperimentalBanks: map['showExperimentalBanks'],
        showCloseButton: map['showCloseButton'],
        showAddButton: map['showAddButton'],
        postSuccessfulConnectionLoadingText:
            map['postSuccessfulConnectionLoadingText'],
        showTransferErrorResult: map['showTransferErrorResult'],
        showTransferSuccessfulResult: map['showTransferSuccessfulResult'],
        theme: DapiThemeConfigurations._fromMap(map['theme']),
        language: language);
  }
}

enum DapiLanguage { EN, AR }

enum DapiTheme { LIGHT, DARK, DYNAMIC }

class DapiColor {
  String? lightMode;
  String? darkMode;

  DapiColor({this.lightMode, this.darkMode});

  static DapiColor _fromMap(Map map) {
    return DapiColor(lightMode: map['lightMode'], darkMode: map['darkMode']);
  }
}

class DapiThemeConfigurations {
  DapiTheme? enforceTheme = DapiTheme.DYNAMIC;
  DapiColor? primaryColor = new DapiColor();

  DapiThemeConfigurations({this.enforceTheme, this.primaryColor});

  static DapiThemeConfigurations _fromMap(Map map) {
    String enforceThemeRaw = map['enforceTheme'].toString().toLowerCase();
    DapiTheme theme;
    if (enforceThemeRaw == "dark") {
      theme = DapiTheme.DARK;
    } else if (enforceThemeRaw == "light") {
      theme = DapiTheme.LIGHT;
    } else {
      theme = DapiTheme.DYNAMIC;
    }
    return DapiThemeConfigurations(
        enforceTheme: theme,
        primaryColor: DapiColor._fromMap(map['primaryColor']));
  }
}

enum DapiEnvironment { PRODUCTION, SANDBOX }

enum DapiTransactionsType { ENRICHED, CATEGORIZED, DEFAULT }

class DapiConnection {
  final String userID;
  final String clientUserID;
  final String bankID;
  final String? swiftCode;
  final String bankShortName;
  final String bankFullName;
  final String? country;
  final String fullLogo;
  final String halfLogo;
  final String miniLogo;

  DapiConnection(
      {required this.userID,
      required this.clientUserID,
      required this.bankID,
      required this.swiftCode,
      required this.bankShortName,
      required this.bankFullName,
      required this.country,
      required this.fullLogo,
      required this.halfLogo,
      required this.miniLogo});

  static DapiConnection _fromMap(Map map) {
    return DapiConnection(
        userID: map['userID'],
        clientUserID: map['clientUserID'],
        bankID: map['bankID'],
        swiftCode: map['swiftCode'],
        bankShortName: map['bankShortName'],
        bankFullName: map['bankFullName'],
        country: map['country'],
        fullLogo: map['fullLogo'],
        halfLogo: map['halfLogo'],
        miniLogo: map['miniLogo']);
  }

  static Future<DapiConnection> create(String jsonConnectionParams) async {
    return await Dapi._createConnection(jsonConnectionParams);
  }

  Future<String> getParameters() async {
    return await Dapi._getParameters(this.userID);
  }

  Future<String> presentAccountSelection() async {
    return await Dapi._presentAccountSelection(this.userID);
  }

  Future<DapiIdentityResponse> getIdentity() async {
    return await Dapi._getIdentity(this.userID);
  }

  Future<DapiAccountsResponse> getAccounts() async {
    return await Dapi._getAccounts(this.userID);
  }

  Future<DapiCardsResponse> getCards() async {
    return await Dapi._getCards(this.userID);
  }

  Future<DapiTransactionsResponse> getTransactionsForAccount(
      DapiAccount account, DateTime fromDate, DateTime toDate,
      {DapiTransactionsType type = DapiTransactionsType.DEFAULT}) async {
    return await Dapi._getTransactionsForAccount(
        this.userID, account.id!, fromDate, toDate, type);
  }

  Future<DapiTransactionsResponse> getTransactionsForCard(
      DapiCard card, DateTime fromDate, DateTime toDate,
      {DapiTransactionsType type = DapiTransactionsType.DEFAULT}) async {
    return await Dapi._getTransactionsForCard(
        this.userID, card.id!, fromDate, toDate, type);
  }

  Future<DapiAccountsMetadataResponse> getAccountsMetadata() async {
    return await Dapi._getAccountsMetadata(this.userID);
  }

  Future<DapiResult> delete() async {
    return await Dapi._delete(this.userID);
  }

  Future<DapiBeneficiariesResponse> getBeneficiaries() async {
    return await Dapi._getBeneficiaries(this.userID);
  }

  Future<DapiResult> createBeneficiary(DapiBeneficiary beneficiary) async {
    return await Dapi._createBeneficiary(this.userID, beneficiary);
  }

  Future<CreateTransferResponse> createTransfer(
    DapiAccount? account,
    DapiBeneficiary? beneficiary,
    double? amount,
    String? remark,
  ) async {
    return await Dapi._createTransfer(
        this.userID, account?.id, beneficiary, amount, remark);
  }

  Future<CreateTransferResponse> createTransferToExistingBeneficiary(
    DapiAccount account,
    String beneficiaryID,
    double amount,
    String? remark,
  ) async {
    return await Dapi._createTransferToExistingBeneficiary(
        this.userID, account.id!, beneficiaryID, amount, remark);
  }

  Future<DapiWireBeneficiariesResponse> getWireBeneficiaries() async {
    return await Dapi._getWireBeneficiaries(this.userID);
  }

  Future<DapiResult> createWireBeneficiary(
      DapiWireBeneficiary beneficiary) async {
    return await Dapi._createWireBeneficiary(this.userID, beneficiary);
  }

  Future<CreateTransferResponse> createWireTransfer(
    DapiAccount? account,
    DapiWireBeneficiary beneficiary,
    double? amount,
    String? remark,
  ) async {
    return await Dapi._createWireTransfer(
        this.userID, account?.id, beneficiary, amount, remark);
  }

  Future<CreateTransferResponse> createWireTransferToExistingBeneficiary(
    DapiAccount account,
    String beneficiaryID,
    double amount,
    String? remark,
  ) async {
    return await Dapi._createWireTransferToExistingBeneficiary(
        this.userID, account.id!, beneficiaryID, amount, remark);
  }

  Future<CreateTransferResponse> createACHPullTransfer(
    String description,
    DapiAccount? account,
    double? amount,
  ) async {
    return await Dapi._createACHPullTransfer(
        this.userID, description, account?.id, amount);
  }
}

class DapiIdentityResponse {
  final String? operationID;
  final DapiIdentity? identity;

  DapiIdentityResponse({this.operationID, this.identity});

  static DapiIdentityResponse _fromMap(Map map) {
    return DapiIdentityResponse(
        operationID: map['operationID'],
        identity: DapiIdentity._fromMap(map["identity"]));
  }
}

class DapiIdentity {
  final String? nationality;
  final String? dateOfBirth;
  final List<DapiNumber>? numbers;
  final String? emailAddress;
  final String? name;
  final DapiAddress? address;
  final List<DapiIdentification>? identifications;

  DapiIdentity(
      {this.nationality,
      this.dateOfBirth,
      this.numbers,
      this.emailAddress,
      this.name,
      this.address,
      this.identifications});

  static DapiIdentity _fromMap(Map map) {
    Map<String, dynamic> addressMap = Map<String, dynamic>.from(map['address']);
    List<dynamic>? numbersMap;
    List<dynamic>? identificationsMap;

    DapiAddress? address;
    List<DapiNumber>? numbers = [];
    List<DapiIdentification>? identifications = [];

    try {
      numbersMap = List<dynamic>.from(map['numbers']);
    } catch (e) {}

    try {
      identificationsMap = List<dynamic>.from(map['identification']);
    } catch (e) {}

    if (addressMap.isNotEmpty) {
      address = DapiAddress._fromMap(addressMap);
    }

    if (numbersMap != null && numbersMap.isNotEmpty) {
      numbersMap.forEach((mapElement) {
        Map map = mapElement as Map;
        numbers.add(DapiNumber._fromMap(map));
      });
    }

    if (identificationsMap != null && identificationsMap.isNotEmpty) {
      identificationsMap.forEach((mapElement) {
        Map map = mapElement as Map;
        identifications.add(DapiIdentification._fromMap(map));
      });
    }

    return DapiIdentity(
        nationality: map['nationality'],
        dateOfBirth: map['dateOfBirth'],
        numbers: numbers,
        emailAddress: map['emailAddress'],
        name: map['name'],
        address: address,
        identifications: identifications);
  }
}

class DapiAccountsResponse {
  final String? operationID;
  final List<DapiAccount>? accounts;

  DapiAccountsResponse({this.operationID, this.accounts});

  static DapiAccountsResponse _fromMap(Map map) {
    List<DapiAccount> accounts = [];
    if (map.containsKey("accounts")) {
      List<dynamic> accountsMap = map['accounts'];
      accountsMap.forEach((mapElement) {
        Map map = mapElement as Map;
        accounts.add(DapiAccount._fromMap(map));
      });
    }
    return DapiAccountsResponse(
        operationID: map['operationID'], accounts: accounts);
  }
}

class DapiAccount {
  final String? iban;
  final String? number;
  final DapiNamePair? currency;
  final String? type;
  final String? name;
  final String? id;
  final num? balance;

  DapiAccount(
      {this.iban,
      this.number,
      this.currency,
      this.type,
      this.name,
      this.id,
      this.balance});

  static DapiAccount _fromMap(Map map) {
    Map<String, dynamic> currencyMap = {};
    if (map.containsKey("currency")) {
      currencyMap = Map<String, dynamic>.from(map['currency']);
    }

    Map<String, dynamic> balanceMap = {};
    if (map.containsKey("balance")) {
      balanceMap = Map<String, dynamic>.from(map['balance']);
    }

    return DapiAccount(
        iban: map['iban'],
        number: map['number'],
        currency: DapiNamePair._fromMap(currencyMap),
        type: map['type'],
        name: map['name'],
        id: map['id'],
        balance: balanceMap['amount'] != null ? balanceMap['amount'] : 0);
  }
}

class DapiCardsResponse {
  final String? operationID;
  final List<DapiCard>? cards;

  DapiCardsResponse({this.operationID, this.cards});

  static DapiCardsResponse _fromMap(Map map) {
    List<DapiCard> cards = [];
    if (map.containsKey("cards")) {
      List<dynamic> cardsMap = map['cards'];
      cardsMap.forEach((mapElement) {
        Map map = mapElement as Map;
        cards.add(DapiCard._fromMap(map));
      });
    }

    return DapiCardsResponse(operationID: map['operationID'], cards: cards);
  }
}

class DapiCard {
  final String? cardNumber;
  final num? creditLimit;
  final DapiNamePair? currency;
  final String? type;
  final String? name;
  final String? id;
  final String? expiryDate;
  final String? status;
  final CreditBalance? balance;

  DapiCard(
      {this.cardNumber,
      this.creditLimit,
      this.currency,
      this.type,
      this.name,
      this.id,
      this.expiryDate,
      this.status,
      this.balance});

  static DapiCard _fromMap(Map map) {
    Map<String, dynamic> currencyMap = {};
    if (map.containsKey("currency")) {
      currencyMap = Map<String, dynamic>.from(map['currency']);
    }

    Map<String, dynamic> balanceMap = {};
    if (map.containsKey("balance")) {
      balanceMap = Map<String, dynamic>.from(map['balance']);
    }
    return DapiCard(
        cardNumber: map['cardNumber'],
        creditLimit: map['creditLimit'],
        currency: DapiNamePair._fromMap(currencyMap),
        type: map['type'],
        name: map['name'],
        id: map['id'],
        expiryDate: map['expiryDate'],
        status: map['status'],
        balance: CreditBalance._fromMap(balanceMap));
  }
}

class DapiTransactionsResponse {
  final String? operationID;
  final List<DapiTransaction>? transactions;

  DapiTransactionsResponse({this.operationID, this.transactions});

  static DapiTransactionsResponse _fromMap(Map map) {
    List<DapiTransaction> transactions = [];
    if (map.containsKey("transactions")) {
      List<dynamic> transactionsMap = map['transactions'];
      transactionsMap.forEach((mapElement) {
        Map map = mapElement as Map;
        transactions.add(DapiTransaction._fromMap(map));
      });
    }

    return DapiTransactionsResponse(
        operationID: map["operationID"], transactions: transactions);
  }
}

class DapiTransaction {
  final double? beforeAmount;
  final double? afterAmount;
  final double? amount;
  final DapiNamePair? currency;
  final DateTime? date;
  final String? description;
  final String? details;
  final String? type;
  final String? category;
  final DapiBrandDetails? brandDetails;

  DapiTransaction(
      {this.beforeAmount,
      this.afterAmount,
      this.amount,
      this.currency,
      this.date,
      this.description,
      this.details,
      this.type,
      this.category,
      this.brandDetails});

  static DapiTransaction _fromMap(Map map) {
    DapiNamePair? currency;
    if (map.containsKey("currency")) {
      Map<String, dynamic> currencyMap =
          Map<String, dynamic>.from(map['currency']);
      currency = DapiNamePair._fromMap(currencyMap);
    }

    DapiBrandDetails? brandDetails;
    if (map.containsKey("brandDetails")) {
      Map<String, dynamic>? brandDetailsMap =
          Map<String, dynamic>.from(map['brandDetails']);
      brandDetails = DapiBrandDetails._fromMap(brandDetailsMap);
    }

    DateTime? date;
    if (map.containsKey("date")) {
      String stringDate = map['date'];
      date = DateFormat("yyyy-MM-dd'T'HH:mm:ss.SSSZ").parse(stringDate);
    }

    return DapiTransaction(
        beforeAmount: map['beforeAmount'].toDouble(),
        afterAmount: map['afterAmount'].toDouble(),
        amount: map['amount'].toDouble(),
        currency: currency,
        date: date,
        description: map['description'],
        details: map['details'],
        type: map['type'],
        category: map['category'],
        brandDetails: brandDetails);
  }
}

class DapiAccountsMetadataResponse {
  final String? operationID;
  final DapiAccountsMetadata? metadata;

  DapiAccountsMetadataResponse({this.operationID, this.metadata});

  static DapiAccountsMetadataResponse _fromMap(Map map) {
    return DapiAccountsMetadataResponse(
        operationID: map['operationID'],
        metadata: DapiAccountsMetadata._fromMap(map["accountsMetadata"]));
  }
}

class DapiAccountsMetadata {
  final String? swiftCode;
  final String? bankID;
  final String? sortCode;
  final String? bankName;
  final String? branchName;
  final String? branchAddress;
  final DapiValuePair? beneficiaryCoolDownPeriod;
  final DapiValuePair? transactionRange;
  final DapiNamePair? country;
  final bool? isCreateBeneficiaryEndpointRequired;
  final bool? willNewlyAddedBeneficiaryExistBeforeCoolDownPeriod;
  final DapiLineAddress? address;
  final Validators? validators;

  DapiAccountsMetadata(
      {this.swiftCode,
      this.bankID,
      this.sortCode,
      this.bankName,
      this.branchName,
      this.branchAddress,
      this.beneficiaryCoolDownPeriod,
      this.transactionRange,
      this.country,
      this.isCreateBeneficiaryEndpointRequired,
      this.willNewlyAddedBeneficiaryExistBeforeCoolDownPeriod,
      this.address,
      this.validators});

  static DapiAccountsMetadata _fromMap(Map map) {
    Map<String, dynamic>? beneficiaryCoolDownPeriodMap;
    if (map.containsKey("beneficiaryCoolDownPeriod")) {
      beneficiaryCoolDownPeriodMap =
          Map<String, dynamic>.from(map['beneficiaryCoolDownPeriod']);
    }

    Map<String, dynamic>? transactionRangeMap;
    if (map.containsKey("transactionRange")) {
      transactionRangeMap = Map<String, dynamic>.from(map['transactionRange']);
    }

    Map<String, dynamic>? countryMap;
    if (map.containsKey("country")) {
      countryMap = Map<String, dynamic>.from(map['country']);
    }

    Map<String, String>? addressMap;
    if (map.containsKey("address")) {
      addressMap = Map<String, String>.from(map['address']);
    }

    Map<String, dynamic>? validatorsMap;
    if (map.containsKey("validators")) {
      validatorsMap = Map<String, dynamic>.from(map['validators']);
    }

    return DapiAccountsMetadata(
        swiftCode: map['swiftCode'],
        bankID: map['bankId'],
        sortCode: map['sortCode'],
        bankName: map['bankName'],
        branchName: map['branchName'],
        branchAddress: map['branchAddress'],
        beneficiaryCoolDownPeriod: beneficiaryCoolDownPeriodMap != null
            ? DapiValuePair._fromMap(beneficiaryCoolDownPeriodMap)
            : null,
        transactionRange: transactionRangeMap != null
            ? DapiValuePair._fromMap(transactionRangeMap)
            : null,
        country: countryMap != null ? DapiNamePair._fromMap(countryMap) : null,
        isCreateBeneficiaryEndpointRequired:
            map['isCreateBeneficiaryEndpointRequired'],
        willNewlyAddedBeneficiaryExistBeforeCoolDownPeriod:
            map['willNewlyAddedBeneficiaryExistBeforeCoolDownPeriod'],
        address:
            addressMap != null ? DapiLineAddress._fromMap(addressMap) : null,
        validators:
            validatorsMap != null ? Validators._fromMap(validatorsMap) : null);
  }
}

class Validators {
  final CreateBeneficiary? createBeneficiary;
  final CreateTransfer? createTransfer;

  Validators({this.createBeneficiary, this.createTransfer});

  static _fromMap(Map<dynamic, dynamic> map) {
    return Validators(
      createBeneficiary: map['createBeneficiary'] != null
          ? CreateBeneficiary._fromMap(map['createBeneficiary'])
          : null,
      createTransfer: map['createTransfer'] != null
          ? CreateTransfer._fromMap(map['createTransfer'])
          : null,
    );
  }
}

class CreateBeneficiary {
  final CreateBeneficiaryLocalValidation? local;
  final CreateBeneficiarySameValidation? same;

  CreateBeneficiary({this.local, this.same});

  static _fromMap(Map<dynamic, dynamic> map) {
    return CreateBeneficiary(
      local: map['local'] != null
          ? CreateBeneficiaryLocalValidation._fromMap(map['local'])
          : null,
      same: map['same'] != null
          ? CreateBeneficiarySameValidation._fromMap(map['same'])
          : null,
    );
  }
}

class CreateTransfer {
  final CreateTransferLocalValidation? local;
  final CreateTransferSameValidation? same;

  CreateTransfer({this.local, this.same});

  static _fromMap(Map<dynamic, dynamic> map) {
    return CreateTransfer(
      local: map['local'] != null
          ? CreateTransferLocalValidation._fromMap(map['local'])
          : null,
      same: map['same'] != null
          ? CreateTransferSameValidation._fromMap(map['same'])
          : null,
    );
  }
}

class CreateBeneficiaryLocalValidation {
  final AddressValidation? address;
  final Validation? branchAddress;
  final Validation? branchName;
  final Validation? country;
  final NameValidation? name;
  final Validation? nickname;
  final Validation? phoneNumber;
  final Validation? sortCode;
  final Validation? swiftCode;

  CreateBeneficiaryLocalValidation({
    this.address,
    this.branchAddress,
    this.branchName,
    this.country,
    this.name,
    this.nickname,
    this.phoneNumber,
    this.sortCode,
    this.swiftCode,
  });

  static _fromMap(Map<dynamic, dynamic> map) {
    return CreateBeneficiaryLocalValidation(
      address: map['address'] != null
          ? AddressValidation._fromMap(map['address'])
          : null,
      branchAddress: map['branchAddress'] != null
          ? Validation._fromMap(map['branchAddress'])
          : null,
      branchName: map['branchName'] != null
          ? Validation._fromMap(map['branchName'])
          : null,
      country:
          map['country'] != null ? Validation._fromMap(map['country']) : null,
      name: map['name'] != null ? NameValidation._fromMap(map['name']) : null,
      nickname:
          map['nickname'] != null ? Validation._fromMap(map['nickname']) : null,
      phoneNumber: map['phoneNumber'] != null
          ? Validation._fromMap(map['phoneNumber'])
          : null,
      sortCode:
          map['sortCode'] != null ? Validation._fromMap(map['sortCode']) : null,
      swiftCode: map['swiftCode'] != null
          ? Validation._fromMap(map['swiftCode'])
          : null,
    );
  }
}

class CreateBeneficiarySameValidation {
  final AddressValidation? address;
  final Validation? branchAddress;
  final Validation? branchName;
  final Validation? country;
  final NameValidation? name;
  final Validation? nickname;
  final Validation? phoneNumber;
  final Validation? sortCode;
  final Validation? swiftCode;

  CreateBeneficiarySameValidation({
    this.address,
    this.branchAddress,
    this.branchName,
    this.country,
    this.name,
    this.nickname,
    this.phoneNumber,
    this.sortCode,
    this.swiftCode,
  });

  static _fromMap(Map<dynamic, dynamic> map) {
    return CreateBeneficiarySameValidation(
      address: map['address'] != null
          ? AddressValidation._fromMap(map['address'])
          : null,
      branchAddress: map['branchAddress'] != null
          ? Validation._fromMap(map['branchAddress'])
          : null,
      branchName: map['branchName'] != null
          ? Validation._fromMap(map['branchName'])
          : null,
      country:
          map['country'] != null ? Validation._fromMap(map['country']) : null,
      name: map['name'] != null ? NameValidation._fromMap(map['name']) : null,
      nickname:
          map['nickname'] != null ? Validation._fromMap(map['nickname']) : null,
      phoneNumber: map['phoneNumber'] != null
          ? Validation._fromMap(map['phoneNumber'])
          : null,
      sortCode:
          map['sortCode'] != null ? Validation._fromMap(map['sortCode']) : null,
      swiftCode: map['swiftCode'] != null
          ? Validation._fromMap(map['swiftCode'])
          : null,
    );
  }
}

class CreateTransferLocalValidation {
  final Validation? remarks;

  CreateTransferLocalValidation({this.remarks});

  static _fromMap(Map<dynamic, dynamic> map) {
    return CreateTransferLocalValidation(
      remarks:
          map['remarks'] != null ? Validation._fromMap(map['remarks']) : null,
    );
  }
}

class CreateTransferSameValidation {
  final Validation? remarks;

  CreateTransferSameValidation({this.remarks});

  static _fromMap(Map<dynamic, dynamic> map) {
    return CreateTransferSameValidation(
      remarks:
          map['remarks'] != null ? Validation._fromMap(map['remarks']) : null,
    );
  }
}

class NameValidation {
  final String? allowedCharacters;
  final List<dynamic>? attributes;
  final num? length;
  final bool? required;

  NameValidation({
    this.allowedCharacters,
    this.attributes,
    this.length,
    this.required,
  });

  static _fromMap(Map<dynamic, dynamic> map) {
    return NameValidation(
      allowedCharacters:
          map['allowedCharacters'] != null ? map['allowedCharacters'] : null,
      attributes: map['attributes'] != null ? map['attributes'] : null,
      length: map['length'] != null ? map['length'] : null,
      required: map['required'] != null ? map['required'] : null,
    );
  }
}

class AddressValidation {
  final num? length;
  final Validation? line1;
  final Validation? line2;
  final Validation? line3;

  AddressValidation({
    this.length,
    this.line1,
    this.line2,
    this.line3,
  });

  static _fromMap(Map<dynamic, dynamic> map) {
    return AddressValidation(
      length: map['length'] != null ? map['length'] : null,
      line1: map['line1'] != null ? Validation._fromMap(map['line1']) : null,
      line2: map['line2'] != null ? Validation._fromMap(map['line2']) : null,
      line3: map['line3'] != null ? Validation._fromMap(map['line3']) : null,
    );
  }
}

class Validation {
  final String? allowedCharacters;
  final num? length;
  final bool? required;

  Validation({this.allowedCharacters, this.length, this.required});

  static _fromMap(Map<dynamic, dynamic> map) {
    return Validation(
      allowedCharacters:
          map['allowedCharacters'] != null ? map['allowedCharacters'] : null,
      length: map['length'] != null ? map['length'] : null,
      required: map['required'] != null ? map['required'] : null,
    );
  }
}

class DapiWireBeneficiariesResponse {
  final String? operationID;
  final List<WireBeneficiary>? beneficiaries;

  DapiWireBeneficiariesResponse({this.operationID, this.beneficiaries});

  static DapiWireBeneficiariesResponse _fromMap(Map map) {
    List<WireBeneficiary> beneficiaries = [];
    if (map.containsKey("beneficiaries")) {
      List<dynamic> beneficiariesMap = map['beneficiaries'];
      beneficiariesMap.forEach((mapElement) {
        Map map = mapElement as Map;
        beneficiaries.add(WireBeneficiary._fromMap(map));
      });
    }
    return DapiWireBeneficiariesResponse(
        operationID: map["operationID"], beneficiaries: beneficiaries);
  }
}

class WireBeneficiary {
  final String? name;
  final String? bankName;
  final String? accountNumber;
  final String? type;
  final String? status;
  final String? id;
  final String? routingNumber;
  final DapiAddress? address;

  WireBeneficiary(
      {this.name,
      this.bankName,
      this.accountNumber,
      this.type,
      this.status,
      this.id,
      this.routingNumber,
      this.address});

  static WireBeneficiary _fromMap(Map map) {
    Map<String, dynamic> addressMap = {};
    if (map.containsKey("address")) {
      addressMap = Map<String, dynamic>.from(map['address']);
    }
    DapiAddress? address;
    if (addressMap.isNotEmpty) {
      address = DapiAddress._fromMap(addressMap);
    }
    return WireBeneficiary(
        name: map['name'],
        accountNumber: map['accountNumber'],
        bankName: map['bankName'],
        type: map['type'],
        status: map['status'],
        id: map['id'],
        routingNumber: map['routingNumber'],
        address: address);
  }
}

class DapiBeneficiariesResponse {
  final String? operationID;
  final List<Beneficiary>? beneficiaries;

  DapiBeneficiariesResponse({this.operationID, this.beneficiaries});

  static DapiBeneficiariesResponse _fromMap(Map map) {
    List<Beneficiary> beneficiaries = [];
    if (map.containsKey("beneficiaries")) {
      List<dynamic> beneficiariesMap = map['beneficiaries'];
      beneficiariesMap.forEach((mapElement) {
        Map map = mapElement as Map;
        beneficiaries.add(Beneficiary._fromMap(map));
      });
    }
    return DapiBeneficiariesResponse(
        operationID: map["operationID"], beneficiaries: beneficiaries);
  }
}

class Beneficiary {
  final String? name;
  final String? accountNumber;
  final String? iban;
  final String? type;
  final String? status;
  final String? id;

  Beneficiary(
      {this.name,
      this.accountNumber,
      this.iban,
      this.type,
      this.status,
      this.id});

  static Beneficiary _fromMap(Map map) {
    return Beneficiary(
      name: map['name'],
      accountNumber: map['accountNumber'],
      iban: map['iban'],
      type: map['type'],
      status: map['status'],
      id: map['id'],
    );
  }
}

class DapiNamePair {
  final String? code;
  final String? name;

  DapiNamePair({this.code, this.name});

  static DapiNamePair _fromMap(Map<String, dynamic> map) {
    return DapiNamePair(code: map['code'], name: map['name']);
  }
}

class DapiBrandDetails {
  final String? logo;
  final String? name;

  DapiBrandDetails({this.logo, this.name});

  static DapiBrandDetails _fromMap(Map<String, dynamic> map) {
    return DapiBrandDetails(logo: map['logo'], name: map['name']);
  }
}

class DapiValuePair {
  final String? unit;
  final num? value;

  DapiValuePair({this.unit, this.value});

  static DapiValuePair _fromMap(Map<String, dynamic> map) {
    return DapiValuePair(unit: map['unit'], value: map['value']);
  }
}

class CreditBalance {
  final num? amountDue;
  final num? availableBalance;
  final String? dueDate;
  final num? outstandingBalance;

  CreditBalance(
      {this.amountDue,
      this.availableBalance,
      this.dueDate,
      this.outstandingBalance});

  static CreditBalance _fromMap(Map<String, dynamic> map) {
    return CreditBalance(
        amountDue: map['amountDue'],
        availableBalance: map['availableBalance'],
        dueDate: map['dueDate'],
        outstandingBalance: map['outstandingBalance']);
  }
}

class DapiNumber {
  final String? type;
  final String? value;

  DapiNumber({this.type, this.value});

  static DapiNumber _fromMap(Map map) {
    return DapiNumber(type: map['type'], value: map['value']);
  }
}

class DapiIdentification {
  final String? type;
  final String? value;

  DapiIdentification({this.type, this.value});

  static DapiIdentification _fromMap(Map map) {
    return DapiIdentification(type: map['type'], value: map['value']);
  }
}

class DapiAddress {
  final String? flat;
  final String? building;
  final String? area;
  final String? poBox;
  final String? city;
  final String? state;
  final String? country;
  final String? full;

  DapiAddress(
      {this.flat,
      this.building,
      this.area,
      this.poBox,
      this.city,
      this.state,
      this.country,
      this.full});

  static DapiAddress _fromMap(Map<String, dynamic> map) {
    return DapiAddress(
      flat: map['flat'],
      building: map['building'],
      area: map['area'],
      poBox: map['poBox'],
      city: map['city'],
      state: map['state'],
      country: map['country'],
      full: map['full'],
    );
  }
}

class DapiBeneficiary {
  final DapiLineAddress? address;
  final String? accountNumber;
  final String? name;
  final String? bankName;
  final String? swiftCode;
  final String? iban;
  final String? phoneNumber;
  final String? country;
  final String? branchAddress;
  final String? branchName;
  final String? nickname;

  DapiBeneficiary(
      {this.address,
      this.accountNumber,
      this.name,
      this.bankName,
      this.swiftCode,
      this.iban,
      this.phoneNumber,
      this.country,
      this.branchAddress,
      this.branchName,
      this.nickname});

  Map<String, dynamic> _toMap() {
    return {
      "address": address?._toMap(),
      "accountNumber": accountNumber,
      "name": name,
      "bankName": bankName,
      "swiftCode": swiftCode,
      "iban": iban,
      "phoneNumber": phoneNumber,
      "country": country,
      "branchAddress": branchAddress,
      "branchName": branchName,
      "nickname": nickname
    };
  }
}

class DapiWireBeneficiary {
  final DapiLineAddress? address;
  final String? name;
  final String? firstName;
  final String? lastName;
  final String? nickname;
  final String? city;
  final String? state;
  final String? country;
  final String? zipCode;
  final String? receiverType;
  final String? receiverAccountType;
  final String? routingNumber;
  final String? accountNumber;

  DapiWireBeneficiary({
    this.address,
    this.name,
    this.firstName,
    this.lastName,
    this.nickname,
    this.city,
    this.state,
    this.country,
    this.zipCode,
    this.receiverType,
    this.receiverAccountType,
    this.routingNumber,
    this.accountNumber,
  });

  Map<String, dynamic> _toMap() {
    return {
      "address": address?._toMap(),
      "name": name,
      "firstName": firstName,
      "lastName": lastName,
      "nickname": nickname,
      "city": city,
      "state": state,
      "country": country,
      "zipCode": zipCode,
      "receiverType": receiverType,
      "receiverAccountType": receiverAccountType,
      "routingNumber": routingNumber,
      "accountNumber": accountNumber
    };
  }
}

class DapiLineAddress {
  final String? line1;
  final String? line2;
  final String? line3;

  DapiLineAddress({this.line1, this.line2, this.line3});

  static DapiLineAddress _fromMap(Map map) {
    return DapiLineAddress(
        line1: map["line1"], line2: map["line2"], line3: map["line3"]);
  }

  Map<String, dynamic> _toMap() {
    return {
      "line1": line1,
      "line2": line2,
      "line3": line3,
    };
  }
}

class CreateTransferResponse {
  final String? operationID;
  final String? accountID;
  final num? amount;
  final String? remark;
  final String? reference;

  CreateTransferResponse({
    this.operationID,
    this.accountID,
    this.amount,
    this.remark,
    this.reference,
  });

  static CreateTransferResponse _fromMap(Map map) {
    return CreateTransferResponse(
      operationID: map['operationID'],
      accountID: map['account'],
      amount: map['amount'],
      remark: map['remark'],
      reference: map['reference'],
    );
  }
}

class DapiResult {
  final String? operationID;

  DapiResult({this.operationID});

  static DapiResult _fromMap(Map map) {
    return DapiResult(operationID: map['operationID']);
  }
}

class ConnectSuccessEvent {
  final String? bankID;
  final String? userID;

  ConnectSuccessEvent({this.bankID, this.userID});

  static ConnectSuccessEvent _fromMap(Map map) {
    return ConnectSuccessEvent(bankID: map['bankID'], userID: map['userID']);
  }
}

class BankRequestEvent {
  final String? bankName;
  final String? iban;

  BankRequestEvent({this.bankName, this.iban});

  static BankRequestEvent _fromMap(Map map) {
    return BankRequestEvent(bankName: map['bankID'], iban: map['userID']);
  }
}

class ConnectFailureEvent {
  final String? bankID;
  final String? error;
  final String? type;

  ConnectFailureEvent({this.bankID, this.error, this.type});

  static ConnectFailureEvent _fromMap(Map map) {
    return ConnectFailureEvent(
        bankID: map['bankID'],
        error: map['error'].toString().split("&&&").lastOrNull,
        type: map['error'].toString().split("&&&").firstOrNull);
  }
}

class TransferUiWillSendEvent {
  final num? amount;
  final String? accountID;

  TransferUiWillSendEvent({this.amount, this.accountID});

  static TransferUiWillSendEvent _fromMap(Map map) {
    return TransferUiWillSendEvent(
        amount: map['amount'], accountID: map['account']);
  }
}

class DapiSdkException implements Exception {
  final String? code;
  final String? message;
  final String? type;
  final String? operationID;
  final DapiValuePair? beneficiaryCoolDownPeriod;

  DapiSdkException(this.code, this.message, this.type, this.operationID,
      this.beneficiaryCoolDownPeriod);

  static DapiSdkException _fromError(PlatformException ex) {
    try {
      dynamic error = jsonDecode(ex.message!);
      String? message = error["message"];
      String? type = error["type"];
      String? operationID = error["operationID"];
      String? unit = error["unit"];
      num? value = error["value"];

      return DapiSdkException(ex.code, message, type, operationID,
          new DapiValuePair(unit: unit, value: value));
    } on FormatException {
      return DapiSdkException(ex.code, ex.message!, null, null, null);
    }
  }

  @override
  String toString() {
    return 'DapiSdkException{code: $code, message: $message, type: $type, operationID: $operationID, beneficiaryCoolDownPeriod: $beneficiaryCoolDownPeriod}';
  }
}
